#include "stdafx.h"

void msdamo (double sa[], double su[], double sf[], double rjud, double dafr, double gsti,
			double te[], double ad[], double *wmol, double *rhod)
/*
	Purpose:

		The subroutine  msdamo  gives  the  density, molecular

		weight and temperature of the upper atmosphere,  using

		the MSIS86 static and dynamic atmospheric model.
		    -      -          -       -           -- 

	Inputs:
		sa[0]
			right ascension of the local point, in radians (0 to 2.*pi)
		sa[1]
			declination (geocentric latitude) of the local point,
			in radians (-pi to pi).
		sa2[2]
			geocentric altitude of the local point in  meters,  in
			the range 85000. to 2000000.m.
		su[0]
			right ascension of the Sun at the date, in radians (0 to 2.*pi).
		su[1]
			Sun declination in radians (-pi to pi)
		sf[0]
			daily solar flux at  10.7 cm,  adjusted for
			the distance between the earth and the  sun
			at the date, in 1e-22 W/m/m/hz.
		sf[1]
			3 month averaged daily flux in 1e-22 w/m/m/hz, adjusted 
			for the Earth-Sun distance.
		sf[2]
			daily amplitude ap (used when sw[8] = -1)
		sf[3]
			3 hr ap index for current time
		sf[4]
			3 hr ap index for 3 hrs before current time
		sf[5]
			3 hr ap index for 6 hrs before current time
		sf[6]
			3 hr ap index for 9 hrs before current time
		sf[7]
			average of eight 3 hr ap indicies from 12 to 33 hrs prior
			to current time
		sf[8]
			average of eight 3 hr ap indicies from 36 to 59 hrs prior
			to current time
		rjud
			modified julian date referred to 1950.0
		dafr
			time (ut) of the day, in seconds.
		gsti
			greenwich sideral time, in radians (0 to 2.*pi),
			at the time dafr of the date rjud.

	Outputs:
		te[0]
			mean exospheric temperature above the  point  in 
			question, as defined by equation  20,  in  refe-
			rence (1), in kelvin.
		te[1]
			local temperature around the point, in kelvin.
		ad[0]
			logarithm in base 10 of the He number-density.
		ad[1]
			logarithm in base 10 of the O2 number-density.
		ad[2]
			logarithm in base 10 of the N2 number-density.
		ad[3]
			logarithm in base 10 of the Ar number-density.
		ad[4]
			logarithm in base 10 of the O  number-density.
		ad[5]
			logarithm in base 10 of the H  number-density.
		wmol
			mean-molecular-weight of the atmosphere  at  the
			point in kg/kgmol.
		rhod
			mean-mass-density of the atmosphere, in kg/m/m/m

	Obs:
		He	helium
		O2	molecular oxygen
		N2	molecular nitrogen
		Ar	argon
		O	atomic oxygen
		H	atomic hydrogen

	Subcalls:
		gts5

	References:
		[1] Hedin, A. E. MSIS-86 Thermospheric Model. In: Jounal of 
		    Geophysical Research, Vol. 92, n. A5, p. 4649-4662, May, 1987.

	Authors:
		Valdemir Carrara       - INPE - S.J.Campos - Brazil

	Date:
		Sep 2011				C version based on Hedin's gts5 function
*/
{
	static int iday;
	static double sec, alt, glat, glong, stl, f107a, f107; 
	static double ap[7], d[8], t[2];
	static double dma[3], tesig, dens;

	djm_inv (int(rjud), dma);
	iday  = int(rjud - djm(1, 1, int(dma[2])));
	sec   = dafr;
	tesig = gsti;
	glong = (sa[0] - tesig)/_rad;
	glat  = sa[1]/_rad;
	alt   = sa[2]/1000.;
	stl   = (sa[0] - su[0] + _pi)*12/_pi;
	f107a = sf[1];
	f107  = sf[0];
	ap[0] = sf[2];
	ap[1] = sf[3];
	ap[2] = sf[4];
	ap[3] = sf[5];
	ap[4] = sf[6];
	ap[5] = sf[7];
	ap[6] = sf[8];
	gts5(iday, sec, alt, glat, glong, stl, f107a, f107, ap, d, te);
	if (d[0] > 1.) ad[0] = log(d[0])/_al10 + 6;	// converted from #/cm3 to #/m3
	else ad[0] = 0.;
	if (d[1] > 1.) ad[4] = log(d[1])/_al10 + 6;
	else ad[4] = 0.;
	if (d[2] > 1.) ad[2] = log(d[2])/_al10 + 6;
	else ad[2] = 0.;
	if (d[3] > 1.) ad[1] = log(d[3])/_al10 + 6;
	else ad[1] = 0.;
	if (d[4] > 1) ad[3] = log(d[4])/_al10 + 6;
	else ad[3] = 0.;
	if (d[6] > 1) ad[5] = log(d[6])/_al10 + 6;
	else ad[5] = 0.;
	dens  = 1000.*d[5];
	*rhod = dens;
	*wmol = 1.e-6*dens*_avog/(d[0] + d[1] + d[2] + d[3] + d[4] + d[6] + d[7]);  
	return;
}

double gst (int mjd, double tsec)
/*

Purpose:
	The function gst furnishes the Greenwich mean
    sidereal time referred to the J2000.0 equator
    and equinox.

Input:
	djm
		modified julian date in days, referred to
		1950.0.
	tsec
		ut1 time complement in seconds.

Output:
	gst
		greenwich sidereal time in radians

Include:
	consta.h

Author:
	Helio Koiti Kuga  		apr/89   	version 1.0
	Valdemir Carrara 		july 2005	(C version)

Ref.:
	Supplement to the Astronomical Almanac - page
	s15, 1983 (The Astronomical Almanac 1984).
*/
{
	static double tsj, tsg0, gs;

	tsj  = (mjd - 18262.5)/36525.;
	tsg0 = 24110.54841 + (8640184.812866 + 9.3104e-2*tsj - 6.2e-6*tsj*tsj) * tsj;
	tsg0 = tsg0*_pi/43200.;
	gs   = fmod(tsg0 + tsec*_earth_rate, _twopi);
	if (gs < 0) gs = gs + _twopi;

	return gs;
}



int djm (int day, int month, int year)
/*

Purpose:
	The function djm furnishes the modified julian
	date with reference to the day, month and year,
	input real parameters at zero hours of the day.

Input:
	day
		day of the month.
	month
		month of the year.
	year
		year

Output:
	djm
		modified julian date in days, referred to
		1950.0.

Remarks:
	This function is optimized for the period between
	the years 1900-2100.

Author:
	Valdemir Carrara 		july 2005	(C version)
*/
{
	return 367*year + day - 712269 + (int)floor(275*month/9.)
     - (int)floor(7*(year + floor((month + 9)/12.))/4.);

} // djm


void djm_inv (int mjd, double dmy[])
{
/*
Purpose:
	The function djm_inv computes the day, month
	and year corresponding to the modified julian
	date djm.

Input:
	djm
		modified julian date in days, referred to
		1950.0.

Output:
	dmy
		[0]		day of the month.
		[1]		month  month of the year.
		[2]		year.

Author:
	Valdemir Carrara 		july 2005	(C version)
*/

	static int y400, y100, y4, y1, d, i;

	static int d1[13] =
	{0, 31, 61, 92, 122, 153, 184, 214, 245, 275, 306, 337, 366};

	y4   = 0;
	y1   = 0;
	d    = mjd + 127775;
	y400 = d/146097;
	d    = d - y400*146097;
	y100 = d/36524;
	d    = d - y100*36524;

	if (y100 > 3) {
		dmy[0] = 29;
		dmy[1] = 2;
		dmy[2] = 1600 + y400*400 + y100*100 + y4*4 + y1;
	} else {
		y4   = d/1461;
		d    = d - y4*1461;
		y1   = d/365;
		if (y1 > 3) {
			dmy[0] = 29;
			dmy[1] = 2;
			dmy[2] = 1600 + y400*400 + y100*100 + y4*4 + y1;
		} else {
			d    = d - y1*365;
			i    = d/32 + 1;
			d    = d + 1;
			while (d1[i] < d) i++;
  			dmy[1] = i + 2;
			dmy[0] = d - d1[i-1];
			dmy[2] = 1600 + y400*400 + y100*100 + y4*4 + y1;
			if (dmy[1] > 12) {
				dmy[1] = dmy[1] - 12;
				dmy[2] = dmy[2] + 1;
			}
		}
	}
	return;
}

double kp_to_ap (double kp)
/*

	Purpose:
		To convert the planetary 3 hour index kp to the
		corresponding planetary amplitude ap.

	Inputs:
		kp
			planetary index kp, in scale of thirds, from
			0 to 9. (double precision, read only).

	Output:
		kp_to_ap
			planetary amplitude ap. (double)

	Author:
		Valdemir Carrara,      27/02/89              ver. 1.0
		Valdemir Carrara,		19/09/11			C Version
*/
{

	if (kp < 4.5)
	{
		if (kp < 2.2)
		{
			if (kp < 1.2)
			{
				if (kp < 0.5)
				{
					if (kp < 0.2)
						return 0.;
					else
						return 2.;
				}
				else
				{
					if (kp < 0.8)
						return 3.;
					else
						return 4.;
				}
			}
			else
			{
				if (kp < 1.8)
				{
					if(kp < 1.5)
						return 5.;
					else
						return 6.;
				}
				else
					return 7.;
			}
		}
		else
		{
			if (kp < 3.5)
			{
				if (kp < 2.8)
				{
					if (kp < 2.5)
						return 9.;
					else
						return 12.;
				}
				else
				{
					if (kp < 3.2)
						return 15.;
					else
						return 18.;
				}
			}
			else
			{
				if (kp < 4.2)
				{
					if (kp < 3.8)
						return 22.;
					else
						return 27.;
				}
				else
					return 32.;
			}
		}
	}
	else
	{
		if (kp < 6.8)
		{
			if (kp < 5.8)
			{
				if (kp < 5.2)
				{
					if (kp < 4.8)
						return 39.;
					else
						return 48.;
				}
				else
				{
					if (kp < 5.5)
						return 56.;
					else
						return 67.;
				}
			}
			else
			{
				if (kp < 6.5)
				{
					if (kp < 6.2)
						return 80.;
					else
						return 94.;
				}
				else
					return 111.;
			}
		}
		else
		{
			if (kp < 8.2)
			{
				if (kp < 7.5)
				{
					if (kp < 7.2)
						return 132.;
					else
						return 154.;
				}
				else
				{
					if (kp < 7.8)
						return 179.;
					else
						return 207.;
				}
			}
			else
			{
				if (kp < 8.8)
				{
					if (kp < 8.5)
						return 236.;
					else
						return 300.;
				}
				else
					return 400.;
			}
		}
	}
	return 0.;
}

double ap_to_kp (double ap)
/*

	Purpose:
		To convert the planetary amplitude ap to the
		corresponding planetary 3 hour index kp.

	Inputs:
		ap
			planetary amplitude ap, from 0 to 400. (double
			read only).

	Output:
		ap_to_kp
			planetary index kp, in scale of thirds, from
			0 to 9. (double).

	Author:
		Valdemir Carrara,      27/02/89              ver. 1.0
		Valdemir Carrara,		19/09/2011			C Version
*/
{
	if (ap < 35.)
	{
		if (ap < 8.0)
		{
			if (ap < 4.5)
			{
				if (ap < 2.5)
				{
					if (ap < 1.0)
						return 0.;
					else
						return 0.3;
				}
				else
				{
					if (ap < 3.5)
						return 0.7;
					else
						return 1.0;
				}
			}
			else
			{
				if (ap < 6.5)
				{
					if (ap < 5.5)
						return 1.3;
					else
						return 1.7;
				}
				else
					return 2.0;
				
			}
		}
		else
		{
			if (ap < 20.)
			{
				if (ap < 13.5)
				{
					if (ap < 10.5)
						return 2.3;
					else
						return 2.7;
				}
				else
				{
					if (ap < 16.5)
						return 3.0;
					else
						return 3.3;
				}
			}
			else
			{
				if (ap < 29.5)
				{
					if (ap < 24.5)
						return 3.7;
					else
						return 4.0;
				}
				else
					return 4.3;
			}
		}
	}
	else
	{
		if (ap < 121.)
		{
			if (ap < 73.5)
			{
				if (ap < 52.0)
				{
					if (ap < 43.5)
						return 4.7;
					else
						return 5.0;
				}
				else
				{
					if (ap < 61.5)
						return 5.3;
					else
						return 5.7;
				}
			}
			else
			{
				if (ap < 102.)
				{
					if (ap < 87.0)
						return 6.0;
					else
						return 6.3;
				}
				else
					return 6.7;
			}
		}
		else
		{
			if (ap < 221.)
			{
				if (ap < 166.)
				{
					if (ap < 143.)
						return 7.0;
					else
						return 7.3;
				}
				else
				{
					if (ap < 193.)
						return 7.7;
					else
						return 8.0;
				}
			}
			else
			{
				if (ap < 350.)
				{
					if (ap < 268.)
						return 8.3;
					else
						return 8.7;
				}
				else
					return 9.0;
			}
		}
	}
	return 0;
}
